@@ -16,14 +16,15 @@ limitations under the License.
 import base64
 import logging
 import os
+import re
 import traceback
 from pathlib import Path
 
-from huggingface_hub import hf_hub_download
+from huggingface_hub import hf_hub_download, HfApi
 from onnxruntime import SessionOptions
 from tqdm import tqdm
 
-from svr_tts.utils import split_text, split_audio, _crossfade, prepare_prosody, mute_fade
+from svr_tts.utils import split_text, split_audio, _crossfade, prepare_prosody, mute_fade, target_duration
 
 """
 Модуль синтеза речи с использованием нескольких моделей ONNX.
@@ -51,9 +52,6 @@ INPUT_SR = 24_000
 OUTPUT_SR = 22_050
 FADE_LEN = int(0.1 * OUTPUT_SR)
 
-logger = logging.getLogger(__name__)
-
-
 class SynthesisInput(NamedTuple):
     """
     Структура входных данных для синтеза речи.
@@ -83,62 +81,170 @@ class SVR_TTS:
     """
 
     REPO_ID = "selectorrrr/svr-tts-large"
-    MODEL_FILES = {
-        "base": "svr_base_v3.onnx",
-        "semantic": "svr_semantic.onnx",
-        "encoder": "svr_encoder_v1.onnx",
-        "style": "svr_style.onnx",
-        "estimator": "svr_estimator.onnx",
-        "vocoder": "svr_vocoder.onnx",
-        "cfe": "svr_cfe.onnx",
-    }
 
-    def __init__(self, api_key, tokenizer_service_url: str = "https://synthvoice.ru/tokenize_batch",
+    def __init__(self, api_key,
+                 tokenizer_service_url: str = "https://synthvoice.ru/tokenize_batch",
                  providers: Sequence[str | tuple[str, dict[Any, Any]]] | None = None,
                  provider_options: Sequence[dict[Any, Any]] | None = None,
                  session_options: SessionOptions | None = None,
-                 timbre_cache_dir='workspace/voices/') -> None:
+                 timbre_cache_dir: str = 'workspace/voices/',
+                 user_models_dir: str | None = None,
+                 reinit_every: int = 32,
+                 dur_norm_low: float = 5.0,
+                 dur_norm_high: float = 16.0) -> None:
         """
-        Инициализация объектов инференс-сессий для всех моделей.
-
-        Аргументы:
-            tokenizer_service_url: URL для REST-сервиса токенизации.
-            providers: список провайдеров для ONNX-моделей (например, CUDA или CPU).
+        reinit_every — после какого количества обработанных current_input
+        переинициализировать onnx-сессии.
+        Если reinit_every <= 0 — реинициализация отключена.
         """
         if providers is None:
             providers = ["CUDAExecutionProvider", "CPUExecutionProvider"]
+
+        self._providers = providers
+        self._provider_options = provider_options
+        self._session_options = session_options
+        self._reinit_every = int(reinit_every)
+        self._processed_since_reinit = 0
+        self.dur_norm_low = dur_norm_low
+        self.dur_norm_high = dur_norm_high
+
         self.tokenizer_service_url = tokenizer_service_url
-        cache_dir = self._get_cache_dir()
+        self._cache_dir = self._get_cache_dir()
         os.environ["TQDM_POSITION"] = "-1"
-        self.base_model = ort.InferenceSession(self._download("base", cache_dir), providers=providers,
-                                               provider_options=provider_options, sess_options=session_options)
-        self.cfe_model = ort.InferenceSession(self._download("cfe", cache_dir), providers=providers,
-                                               provider_options=provider_options, sess_options=session_options)
-        self.semantic_model = ort.InferenceSession(self._download("semantic", cache_dir), providers=providers,
-                                                   provider_options=provider_options, sess_options=session_options)
-        self.encoder_model = ort.InferenceSession(self._download("encoder", cache_dir), providers=providers,
-                                                  provider_options=provider_options, sess_options=session_options)
-        self.style_model = ort.InferenceSession(self._download("style", cache_dir), providers=providers,
-                                                provider_options=provider_options, sess_options=session_options)
-        self.estimator_model = ort.InferenceSession(self._download("estimator", cache_dir), providers=providers,
-                                                    provider_options=provider_options, sess_options=session_options)
-        self.vocoder_model = ort.InferenceSession(self._download("vocoder", cache_dir), providers=providers,
-                                                  provider_options=provider_options, sess_options=session_options)
+
+        self._user_models_dir = Path(user_models_dir).expanduser() if user_models_dir else None
+
+        self._init_sessions()
+
         if api_key:
             api_key = base64.b64encode(api_key.encode('utf-8')).decode('utf-8')
         self.api_key = api_key
+
         self._timbre_cache_dir = Path(os.path.join(timbre_cache_dir, "timbre_cache"))
         self._timbre_cache_dir.mkdir(parents=True, exist_ok=True)
 
+    def _init_sessions(self) -> None:
+        cache_dir = self._cache_dir
+
+        self.base_model      = ort.InferenceSession(
+            self._resolve("base", cache_dir),
+            providers=self._providers,
+            provider_options=self._provider_options,
+            sess_options=self._session_options,
+        )
+        self.cfe_model       = ort.InferenceSession(
+            self._resolve("cfe", cache_dir),
+            providers=self._providers,
+            provider_options=self._provider_options,
+            sess_options=self._session_options,
+        )
+        self.semantic_model  = ort.InferenceSession(
+            self._resolve("semantic", cache_dir),
+            providers=self._providers,
+            provider_options=self._provider_options,
+            sess_options=self._session_options,
+        )
+        self.encoder_model   = ort.InferenceSession(
+            self._resolve("encoder", cache_dir),
+            providers=self._providers,
+            provider_options=self._provider_options,
+            sess_options=self._session_options,
+        )
+        self.style_model     = ort.InferenceSession(
+            self._resolve("style", cache_dir),
+            providers=self._providers,
+            provider_options=self._provider_options,
+            sess_options=self._session_options,
+        )
+        self.estimator_model = ort.InferenceSession(
+            self._resolve("estimator", cache_dir),
+            providers=self._providers,
+            provider_options=self._provider_options,
+            sess_options=self._session_options,
+        )
+        self.vocoder_model   = ort.InferenceSession(
+            self._resolve("vocoder", cache_dir),
+            providers=self._providers,
+            provider_options=self._provider_options,
+            sess_options=self._session_options,
+        )
+
+    def _maybe_reinit_sessions(self) -> None:
+        """
+        Увеличивает счётчик обработанных элементов и при необходимости
+        реинициализирует onnx-сессии.
+        Если self._reinit_every <= 0 — ничего не делает.
+        """
+        if self._reinit_every <= 0:
+            return
+
+        self._processed_since_reinit += 1
+        if self._processed_since_reinit >= self._reinit_every:
+            self._init_sessions()
+            self._processed_since_reinit = 0
+
     def _get_cache_dir(self) -> str:
         cache_dir = user_cache_dir("svr_tts", "SynthVoiceRu")
         os.makedirs(cache_dir, exist_ok=True)
         return cache_dir
 
-    def _download(self, key: str, cache_dir: str) -> str:
-        return hf_hub_download(repo_id=self.REPO_ID, filename=self.MODEL_FILES[key], cache_dir=cache_dir)
+    # ----- единый селектор имени -----
+    @staticmethod
+    def _pick_best_name(key: str, names: list[str]) -> str | None:
+        """
+        Выбираем *.onnx, чьё имя содержит key:
+        - максимальная версия по суффиксу _vN (если нет — v0)
+        - при равенстве версии берём более длинное имя
+        """
+        key_l = key.lower()
+        best_ver = -1
+        best_len = -1
+        best_name: str | None = None
 
-    def _tokenize(self, token_inputs: List[dict]) -> dict:
+        for raw in names:
+            n = raw.split("/")[-1]
+            nl = n.lower()
+            if key_l not in nl or not nl.endswith(".onnx"):
+                continue
+            m = re.search(r"_v(\d+)\.onnx$", nl)
+            ver = int(m.group(1)) if m else 0
+            name_len = len(nl)
+
+            if (ver > best_ver) or (ver == best_ver and name_len > best_len):
+                best_ver, best_len, best_name = ver, name_len, n
+
+        return best_name
+
+    def _resolve(self, key: str, cache_dir: str) -> str:
+        """
+        1) user_models_dir: ищем *.onnx по key с выбором версии (_vN).
+        2) HF: тот же отбор версий среди файлов репозитория.
+        Если нигде не нашли — FileNotFoundError.
+        """
+        logger = logging.getLogger("SVR_TTS")
+
+        # локальные кандидаты
+        if self._user_models_dir:
+            local_names = [p.name for p in self._user_models_dir.glob("*.onnx")]
+            best_local = self._pick_best_name(key, local_names)
+            if best_local:
+                lp = (self._user_models_dir / best_local)
+                if lp.is_file():
+                    resolved = str(lp.resolve())
+                    return resolved
+
+        # HF
+        return self._download(key, cache_dir)
+
+    def _download(self, key: str, cache_dir: str) -> str:
+        files = HfApi().list_repo_files(self.REPO_ID)
+        best = self._pick_best_name(key, files)
+        if not best:
+            raise FileNotFoundError(f"Не нашли модель '{key}' ни локально, ни в HF репозитории {self.REPO_ID}.")
+        path = hf_hub_download(repo_id=self.REPO_ID, filename=best, cache_dir=cache_dir)
+        return path
+
+    def _tokenize(self, token_inputs) -> dict:
         """
         Отправляет данные для токенизации к REST-сервису и возвращает результат.
 
@@ -229,6 +335,7 @@ class SVR_TTS:
         return semantic
 
     def synthesize_batch(self, inputs: List[SynthesisInput],
+                         stress_exclusions: Dict[str, Any] = {},
                          duration_or_speed: float = None,
                          is_speed: bool = False,
                          scaling_min: float = 0.875,
@@ -242,26 +349,30 @@ class SVR_TTS:
             is_speed: True, если задается скорость речи, False если продолжительность.
             scaling_min: минимальный коэффициент масштабирования.
             scaling_max: максимальный коэффициент масштабирования.
+            stress_exclusions: слова исключения для расстановки ударений
 
         Возвращает:
             Список numpy-массивов, каждый из которых представляет сгенерированное аудио.
         """
         synthesized_audios: List[Optional[np.ndarray]] = []
-        token_list = [{"text": inp.text, "stress": inp.stress} for inp in inputs]
-        tokenize_resp = self._tokenize(token_list)
+        items = [{"text": inp.text, "stress": inp.stress} for inp in inputs]
+        tokenize_req = {"items":items, "exclusions": stress_exclusions}
+        tokenize_resp = self._tokenize(tokenize_req)
         # Обработка каждого элемента входных данных
         tqdm_kwargs = tqdm_kwargs or {}
         for idx, current_input in enumerate(
                 tqdm(inputs, desc=tokenize_resp['desc'], **tqdm_kwargs)):
             if not tokenize_resp['tokens'][idx]:
                 synthesized_audios.append(None)
+                self._maybe_reinit_sessions()
                 continue
             timbre_wave = current_input.timbre_wave_24k.astype(np.float32)
             prosody_wave = current_input.prosody_wave_24k.astype(np.float32)
 
             # Если не задана скорость, рассчитаем длительность
             if not is_speed and not duration_or_speed:
-                duration_or_speed = len(prosody_wave) / 24000
+                duration = len(prosody_wave) / 24000
+                duration_or_speed, _ = target_duration(duration, len(current_input.text), self.dur_norm_low, self.dur_norm_high)
 
             # Получение базовых признаков через базовую модель
             wave_24k, _ = \
@@ -321,9 +432,13 @@ class SVR_TTS:
 
             # Объединяем все сегменты в одно аудио
             synthesized_audios.append(np.concatenate(generated_chunks))
+
+            self._maybe_reinit_sessions()
+
         return synthesized_audios
 
-    def synthesize(self, inputs, max_text_len=150, tqdm_kwargs: Dict[str, Any] = None, rtrim_top_db=40):
+    def synthesize(self, inputs, max_text_len=150, tqdm_kwargs: Dict[str, Any] = None, rtrim_top_db=40,
+                   stress_exclusions: Dict[str, Any] = {}):
         split_inputs = []
         mapping = []
 
@@ -341,7 +456,7 @@ class SVR_TTS:
             mapping.append((idx, len(chunks)))
 
         try:
-            all_waves = self.synthesize_batch(split_inputs, tqdm_kwargs=tqdm_kwargs)
+            all_waves = self.synthesize_batch(split_inputs, stress_exclusions, tqdm_kwargs=tqdm_kwargs)
         except Exception as e:
             traceback.print_exc()
             all_waves = [None] * len(split_inputs)
